@@ -194,7 +194,7 @@ set_compress_level(const struct cmdinfo *cip, const char *value)
 
   level = dpkg_options_parse_arg_int(cip, value);
   if (level < 0 || level > 9)
-    badusage(_("invalid compression level for -%c: %ld'"), cip->oshort, level);
+    badusage(_("invalid compression level for -%c: %ld"), cip->oshort, level);
 
   compress_params.level = level;
 }
