@@ -11,7 +11,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::Arch;
 
@@ -93,7 +93,7 @@ my %debarch_to_debtriplet;
 	$gcc_host_gnu_type = get_gcc_host_gnu_type();
 
 	if ($gcc_host_gnu_type eq '') {
-	    warning(_g("Couldn't determine gcc system type, falling back to " .
+	    warning(_g("couldn't determine gcc system type, falling back to " .
 	               'default (native compilation)'));
 	} else {
 	    my (@host_archtriplet) = gnutriplet_to_debtriplet($gcc_host_gnu_type);
@@ -102,7 +102,7 @@ my %debarch_to_debtriplet;
 	    if (defined $host_arch) {
 		$gcc_host_gnu_type = debtriplet_to_gnutriplet(@host_archtriplet);
 	    } else {
-		warning(_g('Unknown gcc system type %s, falling back to ' .
+		warning(_g('unknown gcc system type %s, falling back to ' .
 		           'default (native compilation)'), $gcc_host_gnu_type);
 		$gcc_host_gnu_type = '';
 	    }
@@ -252,11 +252,11 @@ sub read_triplettable()
 
 sub debtriplet_to_gnutriplet(@)
 {
+    my ($abi, $os, $cpu) = @_;
+
     read_cputable();
     read_ostable();
 
-    my ($abi, $os, $cpu) = @_;
-
     return unless defined($abi) && defined($os) && defined($cpu) &&
         exists($cputable{$cpu}) && exists($ostable{"$abi-$os"});
     return join('-', $cputable{$cpu}, $ostable{"$abi-$os"});
@@ -313,10 +313,10 @@ sub debarch_to_multiarch($)
 
 sub debtriplet_to_debarch(@)
 {
-    read_triplettable();
-
     my ($abi, $os, $cpu) = @_;
 
+    read_triplettable();
+
     if (!defined($abi) || !defined($os) || !defined($cpu)) {
 	return;
     } elsif (exists $debtriplet_to_debarch{"$abi-$os-$cpu"}) {
@@ -328,11 +328,11 @@ sub debtriplet_to_debarch(@)
 
 sub debarch_to_debtriplet($)
 {
-    read_triplettable();
-
     local ($_) = @_;
     my $arch;
 
+    read_triplettable();
+
     if (/^linux-([^-]*)/) {
 	# XXX: Might disappear in the future, not sure yet.
 	$arch = $1;
@@ -365,18 +365,19 @@ sub gnutriplet_to_debarch($)
 
 sub debwildcard_to_debtriplet($)
 {
-    local ($_) = @_;
+    my ($arch) = @_;
+    my @tuple = split /-/, $arch, 3;
 
-    if (/any/) {
-	if (/^([^-]*)-([^-]*)-(.*)/) {
-	    return ($1, $2, $3);
-	} elsif (/^([^-]*)-([^-]*)$/) {
-	    return ('any', $1, $2);
+    if (any { $_ eq 'any' } @tuple) {
+	if (scalar @tuple == 3) {
+	    return @tuple;
+	} elsif (scalar @tuple == 2) {
+	    return ('any', @tuple);
 	} else {
-	    return ($_, $_, $_);
+	    return ('any', 'any', 'any');
 	}
     } else {
-	return debarch_to_debtriplet($_);
+	return debarch_to_debtriplet($arch);
     }
 }
 
@@ -388,7 +389,7 @@ sub debarch_to_cpuattrs($)
     if (defined($cpu)) {
         abitable_load();
 
-        return ($abibits{$abi} || $cpubits{$cpu}, $cpuendian{$cpu});
+        return ($abibits{$abi} // $cpubits{$cpu}, $cpuendian{$cpu});
     } else {
         return;
     }
