@@ -12,7 +12,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 package Dpkg::File;
 
@@ -38,13 +38,13 @@ sub file_lock($$) {
     eval 'use File::FcntlLock';
     if ($@) {
         warning(_g('File::FcntlLock not available; using flock which is not NFS-safe'));
-        flock($fh, LOCK_EX) ||
-            syserr(_g('failed to get a write lock on %s'), $filename);
+        flock($fh, LOCK_EX)
+            or syserr(_g('failed to get a write lock on %s'), $filename);
     } else {
         eval q{
             my $fs = File::FcntlLock->new(l_type => F_WRLCK);
-            $fs->lock($fh, F_SETLKW) ||
-                syserr(_g('failed to get a write lock on %s'), $filename);
+            $fs->lock($fh, F_SETLKW)
+                or syserr(_g('failed to get a write lock on %s'), $filename);
         }
     }
 }
