@@ -194,9 +194,8 @@ sub get_symboltempl {
 
 sub set_symbolname {
     my ($self, $name, $templ, $quoted) = @_;
-    unless (defined $name) {
-	$name = $self->{symbol};
-    }
+
+    $name //= $self->{symbol};
     if (!defined $templ && $name =~ /\s/) {
 	$templ = $name;
     }
@@ -253,8 +252,8 @@ sub get_tag_value {
 # tag sets, versioning info (minver and depid))
 sub equals {
     my ($self, $other, %opts) = @_;
-    $opts{versioning} = 1 unless exists $opts{versioning};
-    $opts{tags} = 1 unless exists $opts{tags};
+    $opts{versioning} //= 1;
+    $opts{tags} //= 1;
 
     return 0 if $self->{symbol} ne $other->{symbol};
 
@@ -329,7 +328,7 @@ sub is_pattern {
 
 # Get pattern type if this symbol is a pattern.
 sub get_pattern_type {
-    return $_[0]->{pattern}{type} || '';
+    return $_[0]->{pattern}{type} // '';
 }
 
 # Get (sub)type of the alias pattern. Returns empty string if current
@@ -411,7 +410,7 @@ sub get_symbolspec {
     if ($template_mode) {
 	if ($self->has_tags()) {
 	    $spec .= sprintf('%s%3$s%s%3$s', $self->get_tagspec(),
-		$self->get_symboltempl(), $self->{symbol_quoted} || '');
+		$self->get_symboltempl(), $self->{symbol_quoted} // '');
 	} else {
 	    $spec .= $self->get_symboltempl();
 	}
