@@ -52,26 +52,16 @@ our $CURRENT_MINOR_VERSION = '0';
 sub init_options {
     my ($self) = @_;
     $self->SUPER::init_options();
-    $self->{options}{include_removal} = 0
-        unless exists $self->{options}{include_removal};
-    $self->{options}{include_timestamp} = 0
-        unless exists $self->{options}{include_timestamp};
-    $self->{options}{include_binaries} = 0
-        unless exists $self->{options}{include_binaries};
-    $self->{options}{preparation} = 1
-        unless exists $self->{options}{preparation};
-    $self->{options}{skip_patches} = 0
-        unless exists $self->{options}{skip_patches};
-    $self->{options}{unapply_patches} = 'auto'
-        unless exists $self->{options}{unapply_patches};
-    $self->{options}{skip_debianization} = 0
-        unless exists $self->{options}{skip_debianization};
-    $self->{options}{create_empty_orig} = 0
-        unless exists $self->{options}{create_empty_orig};
-    $self->{options}{auto_commit} = 0
-        unless exists $self->{options}{auto_commit};
-    $self->{options}{ignore_bad_version} = 0
-        unless exists $self->{options}{ignore_bad_version};
+    $self->{options}{include_removal} //= 0;
+    $self->{options}{include_timestamp} //= 0;
+    $self->{options}{include_binaries} //= 0;
+    $self->{options}{preparation} //= 1;
+    $self->{options}{skip_patches} //= 0;
+    $self->{options}{unapply_patches} //= 'auto';
+    $self->{options}{skip_debianization} //= 0;
+    $self->{options}{create_empty_orig} //= 0;
+    $self->{options}{auto_commit} //= 0;
+    $self->{options}{ignore_bad_version} //= 0;
 }
 
 sub parse_cmdline_option {
@@ -495,15 +485,16 @@ sub do_build {
 
     # Handle modified binary files detected by the auto-patch generation
     my $handle_binary = sub {
-        my ($self, $old, $new) = @_;
-        my $relfn = File::Spec->abs2rel($new, $dir);
-        $binaryfiles->new_binary_found($relfn);
-        unless ($include_binaries or $binaryfiles->binary_is_allowed($relfn)) {
-            errormsg(_g('cannot represent change to %s: %s'), $relfn,
+        my ($self, $old, $new, %opts) = @_;
+
+        my $file = $opts{filename};
+        $binaryfiles->new_binary_found($file);
+        unless ($include_binaries or $binaryfiles->binary_is_allowed($file)) {
+            errormsg(_g('cannot represent change to %s: %s'), $file,
                      _g('binary file contents changed'));
             errormsg(_g('add %s in debian/source/include-binaries if you want ' .
                         'to store the modified binary in the debian tarball'),
-                     $relfn);
+                     $file);
             $self->register_error();
         }
     };
@@ -587,7 +578,7 @@ are templates for supplementary fields that you might want to add:
 
 Origin: <vendor|upstream|other>, <url of original patch>
 Bug: <url in upstream bugtracker>
-Bug-Debian: http://bugs.debian.org/<bugnumber>
+Bug-Debian: https://bugs.debian.org/<bugnumber>
 Bug-Ubuntu: https://launchpad.net/bugs/<bugnumber>
 Forwarded: <no|not-needed|url proving that it has been forwarded>
 Reviewed-By: <name and email of someone who approved the patch>
@@ -644,7 +635,7 @@ sub do_commit {
 
     my $binaryfiles = Dpkg::Source::Package::V2::BinaryFiles->new($dir);
     my $handle_binary = sub {
-        my ($self, $old, $new) = @_;
+        my ($self, $old, $new, %opts) = @_;
         my $fn = File::Spec->abs2rel($new, $dir);
         $binaryfiles->new_binary_found($fn);
     };
@@ -670,7 +661,7 @@ sub do_commit {
     mkpath(File::Spec->catdir($dir, 'debian', 'patches'));
     my $patch = $self->register_patch($dir, $tmpdiff, $patch_name);
     my @editors = ('sensible-editor', $ENV{VISUAL}, $ENV{EDITOR}, 'vi');
-    my $editor = first { defined $_ and find_command($_) } @editors;
+    my $editor = first { find_command($_) } @editors;
     if (not $editor) {
         error(_g('cannot find an editor'));
     }
