@@ -17,7 +17,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 use strict;
 use warnings;
@@ -113,7 +113,7 @@ while (@ARGV) {
 	}
     } elsif (m/^--all$/) {
 	$options{all} = undef;
-    } elsif (m/^-(\?|-help)$/) {
+    } elsif (m/^-(?:\?|-help)$/) {
 	usage(); exit(0);
     } elsif (m/^--version$/) {
 	version(); exit(0);
@@ -121,15 +121,14 @@ while (@ARGV) {
 	usageerr(_g("unknown option \`%s'"), $_);
     }
 }
-
-@ARGV && usageerr(_g('takes no non-option arguments'));
+usageerr(_g('takes no non-option arguments')) if @ARGV;
 
 my $count = 0;
 my @fields = changelog_parse(%options);
 foreach my $f (@fields) {
     print "\n" if $count++;
     if ($fieldname) {
-        print $f->{$fieldname} . "\n";
+        print $f->{$fieldname} . "\n" if exists $f->{$fieldname};
     } else {
         print $f->output();
     }
