@@ -3,7 +3,7 @@
  *
  * Copyright © 1995 Ian Jackson <ian@davenant.greenend.org.uk>
  * Copyright © 2000-2002 Wichert Akkerman <wakkerma@debian.org>
- * Copyright © 2006-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006-2014 Guillem Jover <guillem@debian.org>
  * Copyright © 2008 Pierre Habouzit <madcoder@debian.org>
  * Copyright © 2009-2010 Raphaël Hertzog <hertzog@debian.org>
  *
@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -68,7 +68,7 @@ static int opt_force = 0;
 static char *pass_opts[MAX_OPTS];
 static int nb_opts = 0;
 
-#define DPKG_TMP_EXT ".dpkg-tmp"
+#define ALT_TMP_EXT ".dpkg-tmp"
 
 /*
  * Functions.
@@ -252,13 +252,13 @@ pr(char const *fmt, ...)
 static void *
 xmalloc(size_t size)
 {
-	void *r;
+	void *ptr;
 
-	r = malloc(size);
-	if (!r)
+	ptr = malloc(size);
+	if (!ptr)
 		error(_("malloc failed (%zu bytes)"), size);
 
-	return r;
+	return ptr;
 }
 
 static char *
@@ -403,7 +403,7 @@ log_msg(const char *fmt, ...)
 static int
 spawn(const char *prog, const char *args[])
 {
-	pid_t pid, r;
+	pid_t pid, dead_pid;
 	int status;
 
 	pid = fork();
@@ -413,8 +413,8 @@ spawn(const char *prog, const char *args[])
 		execvp(prog, (char *const *)args);
 		syserr(_("unable to execute %s (%s)"), prog, prog);
 	}
-	while ((r = waitpid(pid, &status, 0)) == -1 && errno == EINTR) ;
-	if (r != pid)
+	while ((dead_pid = waitpid(pid, &status, 0)) == -1 && errno == EINTR) ;
+	if (dead_pid != pid)
 		error(_("wait for subprocess %s failed"), prog);
 
 	return status;
@@ -459,21 +459,19 @@ subcall(const char *prog, ...)
 static bool
 rename_mv(const char *src, const char *dst)
 {
-	struct stat st;
+	const char *args[] = { "mv", src, dst, NULL };
+	int rc;
 
-	if (lstat(src, &st) != 0)
+	if (rename(src, dst) == 0)
+		return true;
+	if (errno == ENOENT)
 		return false;
 
-	if (rename(src, dst) != 0) {
-		const char *args[] = { "mv", src, dst, NULL };
-		int r;
-		r = spawn("mv", args);
-		if (WIFEXITED(r) && WEXITSTATUS(r) == 0)
-			return true;
-		return false;
-	}
+	rc = spawn("mv", args);
+	if (WIFEXITED(rc) && WEXITSTATUS(rc) == 0)
+		return true;
 
-	return true;
+	return false;
 }
 
 static void
@@ -634,21 +632,28 @@ struct slave_link {
 	struct slave_link *next;
 	char *name;
 	char *link;
+	bool updated;
 };
 
 struct commit_operation {
 	struct commit_operation *next;
 
 	enum opcode {
-		opcode_nop,
-		opcode_rm,
-		opcode_mv,
+		OPCODE_NOP,
+		OPCODE_RM,
+		OPCODE_MV,
 	} opcode;
 
 	char *arg_a;
 	char *arg_b;
 };
 
+enum alternative_update_reason {
+	ALT_UPDATE_NO,
+	ALT_UPDATE_SLAVE_CHANGED,
+	ALT_UPDATE_LINK_BROKEN,
+};
+
 struct alternative {
 	char *master_name;
 	char *master_link;
@@ -945,7 +950,7 @@ alternative_add_choice(struct alternative *a, struct fileset *fs)
 }
 
 /* slave_name and slave_link must be allocated with malloc */
-static void
+static struct slave_link *
 alternative_add_slave(struct alternative *a, char *slave_name,
                       char *slave_link)
 {
@@ -956,7 +961,7 @@ alternative_add_slave(struct alternative *a, char *slave_name,
 		if (strcmp(sl->name, slave_name) == 0) {
 			free(sl->link);
 			sl->link = xstrdup(slave_link);
-			return;
+			return sl;
 		}
 		if (sl->next == NULL)
 			break;
@@ -966,11 +971,23 @@ alternative_add_slave(struct alternative *a, char *slave_name,
 	new = xmalloc(sizeof(*new));
 	new->name = slave_name;
 	new->link = slave_link;
+	new->updated = false;
 	new->next = NULL;
 	if (sl)
 		sl->next = new;
 	else
 		a->slaves = new;
+
+	return new;
+}
+
+static void
+alternative_copy_slave(struct alternative *a, struct slave_link *sl)
+{
+	struct slave_link *sl_new;
+
+	sl_new = alternative_add_slave(a, xstrdup(sl->name), xstrdup(sl->link));
+	sl_new->updated = sl->updated;
 }
 
 static const char *
@@ -979,6 +996,12 @@ alternative_status_string(enum alternative_status status)
 	return (status == ALT_ST_AUTO) ? "auto" : "manual";
 }
 
+static const char *
+alternative_status_describe(enum alternative_status status)
+{
+	return (status == ALT_ST_AUTO) ? _("auto mode") : _("manual mode");
+}
+
 static void
 alternative_set_status(struct alternative *a, enum alternative_status status)
 {
@@ -1031,8 +1054,8 @@ alternative_remove_choice(struct alternative *a, const char *file)
  */
 
 enum altdb_flags {
-	altdb_lax_parser = 1 << 0,
-	altdb_warn_parser = 1 << 1,
+	ALTDB_LAX_PARSER = 1 << 0,
+	ALTDB_WARN_PARSER = 1 << 1,
 };
 
 struct altdb_context {
@@ -1050,9 +1073,9 @@ altdb_filter_namelist(const struct dirent *entry)
 {
 	if (strcmp(entry->d_name, ".") == 0 ||
 	    strcmp(entry->d_name, "..") == 0 ||
-	    (strlen(entry->d_name) > strlen(DPKG_TMP_EXT) &&
+	    (strlen(entry->d_name) > strlen(ALT_TMP_EXT) &&
 	     strcmp(entry->d_name + strlen(entry->d_name) -
-	            strlen(DPKG_TMP_EXT), DPKG_TMP_EXT) == 0))
+	            strlen(ALT_TMP_EXT), ALT_TMP_EXT) == 0))
 		return 0;
 	return 1;
 }
@@ -1199,10 +1222,8 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 	}
 
 	fs = alternative_get_fileset(a, master_file);
-	if (fs) {
-		free(master_file);
+	if (fs)
 		ctx->bad_format(ctx, _("duplicate path %s"), master_file);
-	}
 
 	if (stat(master_file, &st)) {
 		char *junk;
@@ -1211,7 +1232,7 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 			syserr(_("cannot stat file '%s'"), master_file);
 
 		/* File not found - remove. */
-		if (ctx->flags & altdb_warn_parser)
+		if (ctx->flags & ALTDB_WARN_PARSER)
 			warning(_("alternative %s (part of link group %s) "
 			          "doesn't exist; removing from list of "
 			          "alternatives"), master_file, a->master_name);
@@ -1237,6 +1258,8 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 			ctx->bad_format(ctx,
 			                _("priority of %s is out of range: %s"),
 			                master_file, prio_str);
+		free(prio_str);
+
 		fs = fileset_new(master_file, prio);
 		for (sl = a->slaves; sl; sl = sl->next) {
 			fileset_add_slave(fs, xstrdup(sl->name),
@@ -1244,6 +1267,8 @@ alternative_parse_fileset(struct alternative *a, struct altdb_context *ctx)
 		}
 		alternative_add_choice(a, fs);
 	}
+	free(master_file);
+
 	return true;
 }
 
@@ -1264,28 +1289,27 @@ alternative_load(struct alternative *a, enum altdb_flags flags)
 	}
 	ctx.modified = false;
 	ctx.flags = flags;
-	if (flags & altdb_lax_parser)
+	if (flags & ALTDB_LAX_PARSER)
 		ctx.bad_format = altdb_parse_stop;
 	else
 		ctx.bad_format = altdb_parse_error;
 	xasprintf(&fn, "%s/%s", admdir, a->master_name);
 	ctx.filename = fn;
 
-	/* Verify the alternative exists */
-	if (stat(ctx.filename, &st) == -1) {
+	/* Open the alternative file. */
+	ctx.fh = fopen(ctx.filename, "r");
+	if (ctx.fh == NULL) {
 		if (errno == ENOENT)
 			return false;
-		else
-			syserr(_("cannot stat file '%s'"), ctx.filename);
-	}
-	if (st.st_size == 0) {
-		return false;
+
+		syserr(_("unable to open file '%s'"), ctx.filename);
 	}
 
-	/* Open the database file */
-	ctx.fh = fopen(ctx.filename, "r");
-	if (ctx.fh == NULL)
-		syserr(_("unable to open file '%s'"), ctx.filename);
+	/* Verify the alternative is not empty. */
+	if (fstat(fileno(ctx.fh), &st) == -1)
+		syserr(_("cannot stat file '%s'"), ctx.filename);
+	if (st.st_size == 0)
+		return false;
 
 	/* Start parsing mandatory attributes (link+status) of the alternative */
 	alternative_reset(a);
@@ -1360,7 +1384,7 @@ alternative_save(struct alternative *a)
 
 	/* Write admin file. */
 	xasprintf(&file, "%s/%s", admdir, a->master_name);
-	xasprintf(&filenew, "%s" DPKG_TMP_EXT, file);
+	xasprintf(&filenew, "%s" ALT_TMP_EXT, file);
 
 	ctx.filename = filenew;
 	ctx.fh = fopen(ctx.filename, "w");
@@ -1508,8 +1532,7 @@ alternative_display_user(struct alternative *a)
 	struct fileset *fs;
 	struct slave_link *sl;
 
-	pr("%s - %s", a->master_name,
-	   (a->status == ALT_ST_AUTO) ? _("auto mode") : _("manual mode"));
+	pr("%s - %s", a->master_name, alternative_status_describe(a->status));
 	current = alternative_get_current(a);
 	if (current) {
 		pr(_("  link currently points to %s"), current);
@@ -1542,52 +1565,54 @@ alternative_display_list(struct alternative *a)
 		pr("%s", fs->master_file);
 }
 
+static void
+alternative_print_choice(struct alternative *a, enum alternative_status status,
+                         struct fileset *fs, int idx, int len)
+{
+	const char *current = alternative_get_current(a);
+	int mark;
+
+	if (a->status == status &&
+	    current && strcmp(current, fs->master_file) == 0)
+		mark = '*';
+	else
+		mark = ' ';
+
+	pr("%c %-12d %-*s % -10d %s", mark, idx, len,
+	   fs->master_file, fs->priority, alternative_status_describe(status));
+}
+
 static const char *
 alternative_select_choice(struct alternative *a)
 {
 	const char *current;
 	char *ret, selection[_POSIX_PATH_MAX];
 	struct fileset *best, *fs;
+	int n_choices;
 	int len, idx;
 
+	n_choices = alternative_choices_count(a);
 	current = alternative_get_current(a);
 	best = alternative_get_best(a);
 	assert(best);
 
+	len = 15;
+	for (fs = a->choices; fs; fs = fs->next)
+		len = max(len, (int)strlen(fs->master_file) + 1);
+
 	for (;;) {
-		const char *mark;
-		int n_choices;
-
-		n_choices = alternative_choices_count(a);
-
 		pr(P_("There is %d choice for the alternative %s (providing %s).",
 		      "There are %d choices for the alternative %s (providing %s).",
 		      n_choices), n_choices, a->master_name, a->master_link);
 		printf("\n");
-		len = 15;
-		for (fs = a->choices; fs; fs = fs->next)
-			len = max(len, (int)strlen(fs->master_file) + 1);
+
 		pr("  %-12.12s %-*.*s %-10.10s %s", _("Selection"), len, len,
 		   _("Path"), _("Priority"), _("Status"));
 		pr("------------------------------------------------------------");
-		if (a->status == ALT_ST_AUTO && current &&
-		    strcmp(current, best->master_file) == 0)
-			mark = "*";
-		else
-			mark = " ";
-		pr("%s %-12d %-*s % -10d %s", mark, 0, len, best->master_file,
-		   best->priority, _("auto mode"));
-		idx = 1;
-		for (fs = a->choices; fs; fs = fs->next) {
-			if (a->status == ALT_ST_MANUAL && current &&
-			    strcmp(current, fs->master_file) == 0)
-				mark = "*";
-			else
-				mark = " ";
-			pr("%s %-12d %-*s % -10d %s", mark, idx, len,
-			   fs->master_file, fs->priority, _("manual mode"));
-			idx++;
-		}
+		idx = 0;
+		alternative_print_choice(a, ALT_ST_AUTO, best, idx++, len);
+		for (fs = a->choices; fs; fs = fs->next, idx++)
+			alternative_print_choice(a, ALT_ST_MANUAL, fs, idx, len);
 		printf("\n");
 		printf(_("Press enter to keep the current choice[*], "
 		         "or type selection number: "));
@@ -1669,12 +1694,12 @@ alternative_commit(struct alternative *a)
 
 	for (op = a->commit_ops; op; op = op->next) {
 		switch (op->opcode) {
-		case opcode_nop:
+		case OPCODE_NOP:
 			break;
-		case opcode_rm:
+		case OPCODE_RM:
 			checked_rm(op->arg_a);
 			break;
-		case opcode_mv:
+		case OPCODE_MV:
 			checked_mv(op->arg_a, op->arg_b);
 			break;
 		}
@@ -1753,19 +1778,19 @@ alternative_prepare_install_single(struct alternative *a, const char *name,
 	char *fntmp, *fn;
 
 	/* Create link in /etc/alternatives. */
-	xasprintf(&fntmp, "%s/%s" DPKG_TMP_EXT, altdir, name);
+	xasprintf(&fntmp, "%s/%s" ALT_TMP_EXT, altdir, name);
 	xasprintf(&fn, "%s/%s", altdir, name);
 	checked_rm(fntmp);
 	checked_symlink(file, fntmp);
-	alternative_add_commit_op(a, opcode_mv, fntmp, fn);
+	alternative_add_commit_op(a, OPCODE_MV, fntmp, fn);
 	free(fntmp);
 
 	if (alternative_path_needs_update(linkname, fn)) {
 		/* Create alternative link. */
-		xasprintf(&fntmp, "%s" DPKG_TMP_EXT, linkname);
+		xasprintf(&fntmp, "%s" ALT_TMP_EXT, linkname);
 		checked_rm(fntmp);
 		checked_symlink(fn, fntmp);
-		alternative_add_commit_op(a, opcode_mv, fntmp, linkname);
+		alternative_add_commit_op(a, OPCODE_MV, fntmp, linkname);
 		free(fntmp);
 	}
 	free(fn);
@@ -1805,11 +1830,11 @@ alternative_prepare_install(struct alternative *a, const char *choice)
 		/* Drop unused slave. */
 		xasprintf(&fn, "%s/%s", altdir, sl->name);
 		if (alternative_path_can_remove(sl->link))
-			alternative_add_commit_op(a, opcode_rm, sl->link, NULL);
+			alternative_add_commit_op(a, OPCODE_RM, sl->link, NULL);
 		else
 			warning(_("not removing %s since it's not a symlink"),
 			        sl->link);
-		alternative_add_commit_op(a, opcode_rm, fn, NULL);
+		alternative_add_commit_op(a, OPCODE_RM, fn, NULL);
 		free(fn);
 	}
 }
@@ -1819,19 +1844,19 @@ alternative_remove(struct alternative *a)
 {
 	struct slave_link *sl;
 
-	checked_rm_args("%s" DPKG_TMP_EXT, a->master_link);
+	checked_rm_args("%s" ALT_TMP_EXT, a->master_link);
 	if (alternative_path_can_remove(a->master_link))
 		checked_rm(a->master_link);
 
-	checked_rm_args("%s/%s" DPKG_TMP_EXT, altdir, a->master_name);
+	checked_rm_args("%s/%s" ALT_TMP_EXT, altdir, a->master_name);
 	checked_rm_args("%s/%s", altdir, a->master_name);
 
 	for (sl = a->slaves; sl; sl = sl->next) {
-		checked_rm_args("%s" DPKG_TMP_EXT, sl->link);
+		checked_rm_args("%s" ALT_TMP_EXT, sl->link);
 		if (alternative_path_can_remove(sl->link))
 			checked_rm(sl->link);
 
-		checked_rm_args("%s/%s" DPKG_TMP_EXT, altdir, sl->name);
+		checked_rm_args("%s/%s" ALT_TMP_EXT, altdir, sl->name);
 		checked_rm_args("%s/%s", altdir, sl->name);
 	}
 	/* Drop admin file */
@@ -1839,8 +1864,54 @@ alternative_remove(struct alternative *a)
 }
 
 static bool
-alternative_is_broken(struct alternative *a)
+alternative_has_broken_slave(struct slave_link *sl, struct fileset *fs)
 {
+	if (fileset_can_install_slave(fs, sl->name)) {
+		char *wanted;
+		char *sl_altlnk, *sl_current;
+
+		/* Verify link -> /etc/alternatives/foo */
+		sl_altlnk = areadlink(sl->link);
+		if (!sl_altlnk)
+			return true;
+		xasprintf(&wanted, "%s/%s", altdir, sl->name);
+		if (strcmp(sl_altlnk, wanted) != 0) {
+			free(wanted);
+			free(sl_altlnk);
+			return true;
+		}
+		free(sl_altlnk);
+		/* Verify /etc/alternatives/foo -> file */
+		sl_current = areadlink(wanted);
+		free(wanted);
+		if (!sl_current)
+			return true;
+		if (strcmp(sl_current, fileset_get_slave(fs, sl->name)) != 0) {
+			free(sl_current);
+			return true;
+		}
+		free(sl_current);
+	} else {
+		char *sl_altlnk;
+
+		/* Slave link must not exist. */
+		if (alternative_path_classify(sl->link) != ALT_PATH_MISSING)
+			return true;
+		xasprintf(&sl_altlnk, "%s/%s", altdir, sl->name);
+		if (alternative_path_classify(sl_altlnk) != ALT_PATH_MISSING) {
+			free(sl_altlnk);
+			return true;
+		}
+		free(sl_altlnk);
+	}
+
+	return false;
+}
+
+static enum alternative_update_reason
+alternative_needs_update(struct alternative *a)
+{
+	enum alternative_update_reason reason = ALT_UPDATE_NO;
 	const char *current;
 	char *altlnk, *wanted;
 	struct fileset *fs;
@@ -1849,12 +1920,12 @@ alternative_is_broken(struct alternative *a)
 	/* Check master link */
 	altlnk = areadlink(a->master_link);
 	if (!altlnk)
-		return true;
+		return ALT_UPDATE_LINK_BROKEN;
 	xasprintf(&wanted, "%s/%s", altdir, a->master_name);
 	if (strcmp(altlnk, wanted) != 0) {
 		free(wanted);
 		free(altlnk);
-		return true;
+		return ALT_UPDATE_LINK_BROKEN;
 	}
 	free(wanted);
 	free(altlnk);
@@ -1862,55 +1933,25 @@ alternative_is_broken(struct alternative *a)
 	/* Stop if we have an unmanaged alternative */
 	current = alternative_get_current(a);
 	if (current == NULL)
-		return true;
-
-	if (!alternative_has_choice(a, current))
-		return false;
+		return ALT_UPDATE_LINK_BROKEN;
 
 	fs = alternative_get_fileset(a, current);
 
+	/* Stop if we do not have the choice. */
+	if (fs == NULL)
+		return ALT_UPDATE_NO;
+
 	/* Check slaves */
 	for (sl = a->slaves; sl; sl = sl->next) {
-		if (fileset_can_install_slave(fs, sl->name)) {
-			char *sl_altlnk, *sl_current;
-
-			/* Verify link -> /etc/alternatives/foo */
-			sl_altlnk = areadlink(sl->link);
-			if (!sl_altlnk)
-				return true;
-			xasprintf(&wanted, "%s/%s", altdir, sl->name);
-			if (strcmp(sl_altlnk, wanted) != 0) {
-				free(wanted);
-				free(sl_altlnk);
-				return true;
-			}
-			free(sl_altlnk);
-			/* Verify /etc/alternatives/foo -> file */
-			sl_current = areadlink(wanted);
-			free(wanted);
-			if (!sl_current)
-				return true;
-			if (strcmp(sl_current, fileset_get_slave(fs, sl->name)) != 0) {
-				free(sl_current);
-				return true;
-			}
-			free(sl_current);
-		} else {
-			char *sl_altlnk;
-
-			/* Slave link must not exist. */
-			if (alternative_path_classify(sl->link) != ALT_PATH_MISSING)
-				return true;
-			xasprintf(&sl_altlnk, "%s/%s", altdir, sl->name);
-			if (alternative_path_classify(sl_altlnk) != ALT_PATH_MISSING) {
-				free(sl_altlnk);
-				return true;
-			}
-			free(sl_altlnk);
+		if (alternative_has_broken_slave(sl, fs)) {
+			if (sl->updated)
+				reason = ALT_UPDATE_SLAVE_CHANGED;
+			else
+				return ALT_UPDATE_LINK_BROKEN;
 		}
 	}
 
-	return false;
+	return reason;
 }
 
 struct alternative_map {
@@ -1969,7 +2010,7 @@ alternative_map_load_names(struct alternative_map *alt_map_obj)
 	for (i = 0; i < count; i++) {
 		struct alternative *a_new = alternative_new(table[i]->d_name);
 
-		if (!alternative_load(a_new, altdb_lax_parser)) {
+		if (!alternative_load(a_new, ALTDB_LAX_PARSER)) {
 			alternative_free(a_new);
 			continue;
 		}
@@ -1990,7 +2031,7 @@ alternative_map_load_tree(struct alternative_map *alt_map_links,
 		struct slave_link *sl;
 		struct alternative *a_new = alternative_new(table[i]->d_name);
 
-		if (!alternative_load(a_new, altdb_lax_parser)) {
+		if (!alternative_load(a_new, ALTDB_LAX_PARSER)) {
 			alternative_free(a_new);
 			continue;
 		}
@@ -2200,12 +2241,56 @@ alternative_select_mode(struct alternative *a, const char *current_choice)
 	}
 }
 
+static void
+alternative_evolve_slave(struct alternative *a, const char *cur_choice,
+                         struct slave_link *sl, struct fileset *fs)
+{
+	struct stat st;
+	char *new_file = NULL;
+	const char *old, *new;
+
+	old = alternative_get_slave(a, sl->name)->link;
+	new = sl->link;
+
+	if (cur_choice && strcmp(cur_choice, fs->master_file) == 0) {
+		new_file = xstrdup(fileset_get_slave(fs, sl->name));
+	} else {
+		char *lnk;
+
+		xasprintf(&lnk, "%s/%s", altdir, sl->name);
+		new_file = areadlink(lnk);
+		free(lnk);
+	}
+	if (strcmp(old, new) != 0 &&
+	    alternative_path_classify(old) == ALT_PATH_SYMLINK) {
+		bool rename_link = false;
+
+		if (new_file) {
+			errno = 0;
+			if (stat(new_file, &st) == -1 && errno != ENOENT)
+				syserr(_("cannot stat file '%s'"),
+				       new_file);
+			rename_link = (errno == 0);
+		}
+
+		if (rename_link) {
+			info(_("renaming %s slave link from %s to %s"),
+			     sl->name, old, new);
+			checked_mv(old, new);
+		} else {
+			checked_rm(old);
+		}
+
+		sl->updated = true;
+	}
+	free(new_file);
+}
+
 static void
 alternative_evolve(struct alternative *a, struct alternative *b,
                    const char *cur_choice, struct fileset *fs)
 {
 	struct slave_link *sl;
-	struct stat st;
 	bool is_link;
 
 	is_link = alternative_path_classify(a->master_link) == ALT_PATH_SYMLINK;
@@ -2219,47 +2304,12 @@ alternative_evolve(struct alternative *a, struct alternative *b,
 	/* Check if new slaves have been added, or existing
 	 * ones renamed. */
 	for (sl = b->slaves; sl; sl = sl->next) {
-		char *new_file = NULL;
-		const char *old, *new;
+		if (alternative_has_slave(a, sl->name))
+			alternative_evolve_slave(a, cur_choice, sl, fs);
+		else
+			sl->updated = true;
 
-		if (!alternative_has_slave(a, sl->name)) {
-			alternative_add_slave(a, xstrdup(sl->name),
-			                      xstrdup(sl->link));
-			continue;
-		}
-		old = alternative_get_slave(a, sl->name)->link;
-		new = sl->link;
-		if (cur_choice && strcmp(cur_choice, fs->master_file) == 0) {
-			new_file = xstrdup(fileset_get_slave(fs, sl->name));
-		} else {
-			char *lnk;
-
-			xasprintf(&lnk, "%s/%s", altdir, sl->name);
-			new_file = areadlink(lnk);
-			free(lnk);
-		}
-		if (strcmp(old, new) != 0 &&
-		    alternative_path_classify(old) == ALT_PATH_SYMLINK) {
-			bool rename_link = false;
-
-			if (new_file) {
-				errno = 0;
-				if (stat(new_file, &st) == -1 && errno != ENOENT)
-					syserr(_("cannot stat file '%s'"),
-					       new_file);
-				rename_link = (errno == 0);
-			}
-
-			if (rename_link) {
-				info(_("renaming %s slave link from %s to %s"),
-				     sl->name, old, new);
-				checked_mv(old, new);
-			} else {
-				checked_rm(old);
-			}
-		}
-		free(new_file);
-		alternative_add_slave(a, xstrdup(sl->name), xstrdup(sl->link));
+		alternative_copy_slave(a, sl);
 	}
 }
 
@@ -2267,6 +2317,8 @@ static void
 alternative_update(struct alternative *a,
                    const char *current_choice, const char *new_choice)
 {
+	enum alternative_update_reason reason;
+
 	/* No choice left, remove everything. */
 	if (!alternative_choices_count(a)) {
 		log_msg("link group %s fully removed", a->master_name);
@@ -2281,15 +2333,22 @@ alternative_update(struct alternative *a,
 		        new_choice);
 		info(_("using %s to provide %s (%s) in %s"), new_choice,
 		     a->master_link, a->master_name,
-		     (a->status == ALT_ST_AUTO) ? _("auto mode") :
-		                                  _("manual mode"));
+		     alternative_status_describe(a->status));
 		debug("prepare_install(%s)", new_choice);
 		alternative_prepare_install(a, new_choice);
-	} else if (alternative_is_broken(a)) {
-		log_msg("auto-repair link group %s", a->master_name);
-		warning(_("forcing reinstallation of alternative %s because "
-		          "link group %s is broken"), current_choice,
-		        a->master_name);
+	} else if ((reason = alternative_needs_update(a))) {
+		if (reason == ALT_UPDATE_SLAVE_CHANGED) {
+			log_msg("link group %s updated with changed slaves",
+			        a->master_name);
+			info(_("updating alternative %s "
+			       "because link group %s has changed slave links"),
+			     current_choice, a->master_name);
+		} else {
+			log_msg("auto-repair link group %s", a->master_name);
+			warning(_("forcing reinstallation of alternative %s "
+			          "because link group %s is broken"),
+			        current_choice, a->master_name);
+		}
 
 		if (current_choice && !alternative_has_choice(a, current_choice)) {
 			struct fileset *best = alternative_get_best(a);
@@ -2624,19 +2683,19 @@ main(int argc, char **argv)
 	    strcmp(action, "config") == 0 ||
 	    strcmp(action, "remove-all") == 0) {
 		/* Load the alternative info, stop on failure. */
-		if (!alternative_load(a, altdb_warn_parser))
+		if (!alternative_load(a, ALTDB_WARN_PARSER))
 			error(_("no alternatives for %s"), a->master_name);
 	} else if (strcmp(action, "remove") == 0) {
 		/* FIXME: Be consistent for now with the case when we
 		 * try to remove a non-existing path from an existing
 		 * link group file. */
-		if (!alternative_load(a, altdb_warn_parser)) {
+		if (!alternative_load(a, ALTDB_WARN_PARSER)) {
 			verbose(_("no alternatives for %s"), a->master_name);
 			exit(0);
 		}
 	} else if (strcmp(action, "install") == 0) {
 		/* Load the alternative info, ignore failures. */
-		alternative_load(a, altdb_warn_parser);
+		alternative_load(a, ALTDB_WARN_PARSER);
 	}
 
 	/* Handle actions. */
